@@ -1,7 +1,7 @@
 /*
  * mainconf.c	Handle the server's configuration.
  *
- * Version:	$Id: mainconfig.c,v 1.36.2.6.2.3 2006/11/18 15:03:39 nbk Exp $
+ * Version:	$Id: mainconfig.c,v 1.36.2.6.2.6 2007/02/09 15:56:11 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -1224,6 +1224,26 @@ CONF_SECTION *read_radius_conf_file(void)
 {
 	char buffer[256];
 	CONF_SECTION *cs;
+	struct stat statbuf;
+
+	if (stat(radius_dir, &statbuf) < 0) {
+		radlog(L_ERR|L_CONS, "Errors reading %s: %s",
+		       radius_dir, strerror(errno));
+		return NULL;
+	}
+
+	if ((statbuf.st_mode & S_IWOTH) != 0) {
+		radlog(L_ERR|L_CONS, "Configuration directory %s is globally writable.  Refusing to start due to insecure configuration.",
+		       radius_dir);
+	  return NULL;
+	}
+
+
+	if (0 && (statbuf.st_mode & S_IROTH) != 0) {
+		radlog(L_ERR|L_CONS, "Configuration directory %s is globally readable.  Refusing to start due to insecure configuration.",
+		       radius_dir);
+		return NULL;
+	}
 
 	/* Lets go look for the new configuration files */
 	snprintf(buffer, sizeof(buffer), "%.200s/%.50s", radius_dir, RADIUS_CONFIG);
@@ -1476,6 +1496,14 @@ int free_mainconfig(void)
 	cf_section_free(&mainconfig.config);
 	realm_free(mainconfig.realms);
 	clients_free(mainconfig.clients);
+	read_naslist_file(NULL);
+
+	rl_free();
+	listen_free(mainconfig.listen);
+	paircompare_builtin_free();
+	xlat_free();
+	dict_free();
+	lt_dlexit();
 
 	return 0;
 }
