@@ -1,7 +1,7 @@
 /*
  * modules.c	Radius module support.
  *
- * Version:	$Id: modules.c,v 1.76 2003/05/20 14:13:43 aland Exp $
+ * Version:	$Id: modules.c,v 1.86.2.1.2.5 2007/02/09 10:39:11 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
 
-static const char rcsid[] = "$Id: modules.c,v 1.76 2003/05/20 14:13:43 aland Exp $";
+static const char rcsid[] = "$Id: modules.c,v 1.86.2.1.2.5 2007/02/09 10:39:11 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -128,7 +128,7 @@ static void instance_list_free(module_instance_t **i)
 		next = c->next;
 		if(c->entry->module->detach)
 			(c->entry->module->detach)(c->insthandle);
-#if HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
 		if (c->mutex) {
 			/*
 			 *	FIXME
@@ -218,7 +218,7 @@ static module_list_t *linkto_module(const char *module_name,
 	node->next = NULL;
 	node->handle = handle;
 	strNcpy(node->name, module_name, sizeof(node->name));
-	
+
 	/*
 	 *	Link to the module's rlm_FOO{} module structure.
 	 */
@@ -237,7 +237,7 @@ static module_list_t *linkto_module(const char *module_name,
 		free(node);
 		return NULL;
 	}
-	
+
 	/* call the modules initialization */
 	if (node->module->init && (node->module->init)() < 0) {
 		radlog(L_ERR|L_CONS, "%s[%d] Module initialization failed.\n",
@@ -304,7 +304,7 @@ module_instance_t *find_module_instance(const char *instname)
 	 *	no name2.
 	 */
 	name1 = name2 = NULL;
-	for(inst_cs=cf_subsection_find_next(cs, NULL, NULL); 
+	for(inst_cs=cf_subsection_find_next(cs, NULL, NULL);
 			inst_cs != NULL;
 			inst_cs=cf_subsection_find_next(cs, inst_cs, NULL)) {
 		name1 = cf_section_name1(inst_cs);
@@ -324,7 +324,7 @@ module_instance_t *find_module_instance(const char *instname)
 	node = rad_malloc(sizeof(*node));
 	node->next = NULL;
 	node->insthandle = NULL;
-	
+
 	/*
 	 *	Link to the module by name: rlm_FOO-major.minor
 	 */
@@ -351,7 +351,7 @@ module_instance_t *find_module_instance(const char *instname)
 		/* linkto_module logs any errors */
 		return NULL;
 	}
-	
+
 	/*
 	 *	Call the module's instantiation routine.
 	 */
@@ -371,7 +371,7 @@ module_instance_t *find_module_instance(const char *instname)
 	 */
 	strNcpy(node->name, instname, sizeof(node->name));
 
-#if HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
 	/*
 	 *	If we're threaded, check if the module is thread-safe.
 	 *
@@ -390,11 +390,11 @@ module_instance_t *find_module_instance(const char *instname)
 		node->mutex = NULL;
 	}
 
-#endif	
+#endif
 	*last = node;
 
 	DEBUG("Module: Instantiated %s (%s) ", name1, node->name);
-	
+
 	return node;
 }
 
@@ -419,7 +419,7 @@ static indexed_modcallable *new_sublist(int comp, int idx)
 		/* It is an error to try to create a sublist that already
 		 * exists. It would almost certainly be caused by accidental
 		 * duplication in the config file.
-		 * 
+		 *
 		 * index 0 is the exception, because it is used when we want
 		 * to collect _all_ listed modules under a single index by
 		 * default, which is currently the case in all components
@@ -448,6 +448,8 @@ static int indexed_modcall(int comp, int idx, REQUEST *request)
 
 	this = lookup_by_index(components[comp], idx);
 	if (!this) {
+		if (idx != 0) DEBUG2("  ERROR: Unknown value specified for %s.  Cannot perform requested action.",
+				     subcomponent_names[comp]);
 		/* Return a default value appropriate for the component */
 		switch(comp) {
 			case RLM_COMPONENT_AUTZ:    return RLM_MODULE_NOTFOUND;
@@ -461,11 +463,14 @@ static int indexed_modcall(int comp, int idx, REQUEST *request)
 			default:                    return RLM_MODULE_FAIL;
 		}
 	}
+
+	DEBUG2("  Processing the %s section of radiusd.conf",
+	       component_names[comp]);
 	return modcall(comp, this->modulelist, request);
 }
 
 /* Load a flat module list, as found inside an authtype{} block */
-static void load_subcomponent_section(CONF_SECTION *cs, int comp,
+static int load_subcomponent_section(CONF_SECTION *cs, int comp,
 				      const char *filename)
 {
 	int idx;
@@ -476,6 +481,9 @@ static void load_subcomponent_section(CONF_SECTION *cs, int comp,
 	static int meaningless_counter = 1;
 
 	ml = compile_modgroup(comp, cs, filename);
+	if (!ml) {
+		return 0;
+	}
 
 	/* We must assign a numeric index to this subcomponent. For
 	 * auth, it is generated and placed in the dictionary by
@@ -493,6 +501,10 @@ static void load_subcomponent_section(CONF_SECTION *cs, int comp,
 		dval = dict_valbyname(PW_ACCT_TYPE, cf_section_name2(cs));
 	} else if (comp == RLM_COMPONENT_SESS) {
 		dval = dict_valbyname(PW_SESSION_TYPE, cf_section_name2(cs));
+	} else if (comp == RLM_COMPONENT_PRE_PROXY) {
+		dval = dict_valbyname(PW_PRE_PROXY_TYPE, cf_section_name2(cs));
+	} else if (comp == RLM_COMPONENT_POST_PROXY) {
+		dval = dict_valbyname(PW_POST_PROXY_TYPE, cf_section_name2(cs));
 	} else if (comp == RLM_COMPONENT_POST_AUTH) {
 		dval = dict_valbyname(PW_POST_AUTH_TYPE, cf_section_name2(cs));
 	}
@@ -510,58 +522,102 @@ static void load_subcomponent_section(CONF_SECTION *cs, int comp,
 				filename, cf_section_lineno(cs),
 				subcomponent_names[comp], cf_section_name2(cs));
 		modcallable_free(&ml);
-		return;
+		return 1;
 	}
 
 	subcomp->modulelist = ml;
+
+	return 1;
 }
 
-static void load_component_section(CONF_SECTION *cs, int comp, const char *filename)
+static int load_component_section(CONF_SECTION *cs, int comp,
+				  const char *filename)
 {
 	modcallable *this;
 	CONF_ITEM *modref;
-	int modreflineno;
 	int idx;
 	indexed_modcallable *subcomp;
 	const char *modname;
 	char *visiblename;
 
-	for (modref=cf_item_find_next(cs, NULL); 
+	for (modref=cf_item_find_next(cs, NULL);
 			modref != NULL;
 			modref=cf_item_find_next(cs, modref)) {
+		CONF_PAIR *cp = NULL;
+		CONF_SECTION *scs = NULL;
 
 		if (cf_item_is_section(modref)) {
-			CONF_SECTION *scs;
+			const char *sec_name;
 			scs = cf_itemtosection(modref);
 
-			if (strcmp(cf_section_name1(scs),
+			sec_name = cf_section_name1(scs);
+
+			if (strcmp(sec_name,
 				   subcomponent_names[comp]) == 0) {
-				load_subcomponent_section(scs, comp, filename);
+				if (!load_subcomponent_section(scs, comp,
+							       filename)) {
+					return -1; /* FIXME: memleak? */
+				}
 				continue;
 			}
 
 			/*
 			 *	Allow old names, too.
 			 */
-			if (strcmp(cf_section_name1(scs),
+			if (strcmp(sec_name,
 				   old_subcomponent_names[comp]) == 0) {
-				load_subcomponent_section(scs, comp, filename);
+				if (!load_subcomponent_section(scs, comp,
+							       filename)) {
+					return -1; /* FIXME: memleak? */
+				}
 				continue;
 			}
-			modreflineno = cf_section_lineno(scs);
+			cp = NULL;
 		} else {
-			CONF_PAIR *cp;
 			cp = cf_itemtopair(modref);
-			modreflineno = cf_pair_lineno(cp);
 		}
 
 		this = compile_modsingle(comp, modref, filename, &modname);
+                if (!this) {
+                        radlog(L_ERR|L_CONS,
+                               "%s[%d] Failed to parse %s section.\n",
+                               filename, cf_section_lineno(cs),
+                               cf_section_name1(cs));
+                        return -1;
+                }
 
 		if (comp == RLM_COMPONENT_AUTH) {
 			DICT_VALUE *dval;
+			const char *modrefname = NULL;
+			int lineno = 0;
 
-			dval = dict_valbyname(PW_AUTH_TYPE, modname);
-			rad_assert(dval != NULL);
+			if (cp) {
+				modrefname = cf_pair_attr(cp);
+				lineno = cf_pair_lineno(cp);
+			} else {
+				modrefname = cf_section_name2(scs);
+				lineno = cf_section_lineno(scs);
+
+				if (!modrefname) {
+					radlog(L_ERR|L_CONS,
+					       "%s[%d] Failed to parse %s sub-section.\n",
+					       filename, lineno,
+					       cf_section_name1(scs));
+					return -1;
+				}
+			}
+			
+			dval = dict_valbyname(PW_AUTH_TYPE, modrefname);
+			if (!dval) {
+				/*
+				 *	It's a section, but nothing we
+				 *	recognize.  Die!
+				 */
+				radlog(L_ERR|L_CONS, "%s[%d] Unknown Auth-Type \"%s\" in %s section.",
+				       filename, lineno,
+				       modrefname, component_names[comp]);
+				return -1;
+			}
 			idx = dval->value;
 		} else {
 			/* See the comment in new_sublist() for explanation
@@ -571,7 +627,7 @@ static void load_component_section(CONF_SECTION *cs, int comp, const char *filen
 
 		subcomp = new_sublist(comp, idx);
 		if (subcomp == NULL) {
-			radlog(L_ERR|L_CONS,
+			radlog(L_INFO|L_CONS,
 					"%s %s %s already configured - skipping",
 					filename, subcomponent_names[comp],
 					modname);
@@ -587,6 +643,8 @@ static void load_component_section(CONF_SECTION *cs, int comp, const char *filen
 		add_to_modcallable(&subcomp->modulelist, this,
 				comp, visiblename);
 	}
+
+	return 0;
 }
 
 typedef struct section_type_value_t {
@@ -643,7 +701,7 @@ int setup_modules(void)
 		/*
 		 *	Set the default list of preloaded symbols.
 		 *	This is used to initialize libltdl's list of
-		 *	preloaded modules. 
+		 *	preloaded modules.
 		 *
 		 *	i.e. Static modules.
 		 */
@@ -653,7 +711,7 @@ int setup_modules(void)
 			radlog(L_ERR|L_CONS, "Failed to initialize libraries: %s\n",
 					lt_dlerror());
 			exit(1); /* FIXME */
-			
+
 		}
 
 		/*
@@ -662,7 +720,7 @@ int setup_modules(void)
 		 *	any location on the disk.
 		 */
 		lt_dlsetsearchpath(radlib_dir);
-		
+
 		DEBUG2("Module: Library search path is %s",
 				lt_dlgetsearchpath());
 
@@ -710,11 +768,12 @@ int setup_modules(void)
 						      section_type_value[comp].typename);
 
 			/*
-			 *	Allow old names, too.
+			 *	Allow some old names, too.
 			 */
-			if (!next) {
+			if (!next && (comp <= 4)) {
+
 				next = cf_subsection_find_next(cs, sub,
-							      old_section_type_value[comp].typename);
+							       old_section_type_value[comp].typename);
 			}
 			sub = next;
 
@@ -803,15 +862,18 @@ int setup_modules(void)
 		/*
 		 *  Loop over the items in the 'instantiate' section.
 		 */
-		for (ci=cf_item_find_next(cs, NULL); ci != NULL; ci=cf_item_find_next(cs, ci)) {
+		for (ci=cf_item_find_next(cs, NULL);
+		     ci != NULL;
+		     ci=cf_item_find_next(cs, ci)) {
+
 			if (cf_item_is_section(ci)) {
 				radlog(L_ERR|L_CONS,
 				       "%s[%d] Subsection for module instantiate is not allowed\n", filename,
-				       
+
 				       cf_section_lineno(cf_itemtosection(ci)));
 				exit(1);
 			}
-	
+
 			cp = cf_itemtopair(ci);
 			name = cf_pair_attr(cp);
 			module = find_module_instance(name);
@@ -827,10 +889,12 @@ int setup_modules(void)
 	 */
 	for (comp = 0; comp < RLM_COMPONENT_COUNT; ++comp) {
 		cs = cf_section_find(component_names[comp]);
-		if (cs == NULL) 
+		if (cs == NULL)
 			continue;
-		
-		load_component_section(cs, comp, filename);
+
+		if (load_component_section(cs, comp, filename) < 0) {
+			exit(1);
+		}
 	}
 
 	return 0;
@@ -912,17 +976,17 @@ int module_checksimul(int sess_type, REQUEST *request, int maxsimul)
 /*
  *	Do pre-proxying for ALL configured sessions
  */
-int module_pre_proxy(REQUEST *request)
+int module_pre_proxy(int type, REQUEST *request)
 {
-	return indexed_modcall(RLM_COMPONENT_PRE_PROXY, 0, request);
+	return indexed_modcall(RLM_COMPONENT_PRE_PROXY, type, request);
 }
 
 /*
  *	Do post-proxying for ALL configured sessions
  */
-int module_post_proxy(REQUEST *request)
+int module_post_proxy(int type, REQUEST *request)
 {
-	return indexed_modcall(RLM_COMPONENT_POST_PROXY, 0, request);
+	return indexed_modcall(RLM_COMPONENT_POST_PROXY, type, request);
 }
 
 /*
