@@ -1,7 +1,7 @@
 /*
  * rlm_eap_ttls.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: ttls.c,v 1.17.2.2.2.3 2006/10/18 21:49:47 aland Exp $
+ * Version:     $Id: ttls.c,v 1.17.2.2.2.4 2007/02/16 09:31:34 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -611,6 +611,24 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 			DEBUG2("  TTLS: Got MS-CHAP2-Success, tunneling it to the client in a challenge.");
 			rcode = RLM_MODULE_HANDLED;
 			t->authenticated = TRUE;
+			
+			/*
+			 *	Delete MPPE keys & encryption policy.  We don't
+			 *	want these here.
+			 */
+			pairdelete(&reply->vps, ((311 << 16) | 7));
+			pairdelete(&reply->vps, ((311 << 16) | 8));
+			pairdelete(&reply->vps, ((311 << 16) | 16));
+			pairdelete(&reply->vps, ((311 << 16) | 17));
+			
+			/*
+			 *	Use the tunneled reply, but not now.
+			 */
+			if (t->use_tunneled_reply) {
+				t->reply = reply->vps;
+				reply->vps = NULL;
+			}
+
 		} else { /* no MS-CHAP2-Success */
 			/*
 			 *	Can only have EAP-Message if there's
