@@ -1,7 +1,7 @@
 /*
  * rlm_pap.c
  *
- * Version:  $Id: rlm_pap.c,v 1.15.2.2.2.2 2006/12/19 00:35:58 aland Exp $
+ * Version:  $Id: rlm_pap.c,v 1.15.2.2.2.4 2007/03/05 13:12:29 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@
 #define PAP_MAX_ENC		9
 
 
-static const char rcsid[] = "$Id: rlm_pap.c,v 1.15.2.2.2.2 2006/12/19 00:35:58 aland Exp $";
+static const char rcsid[] = "$Id: rlm_pap.c,v 1.15.2.2.2.4 2007/03/05 13:12:29 aland Exp $";
 
 /*
  *      Define a structure for our module configuration.
@@ -213,8 +213,6 @@ static int base64_decode (const char *src, uint8_t *dst)
 	length = equals = 0;
 	while (src[length] && src[length] != '=') length++;
 
-	if (src[length] != '=') return 0; /* no trailing '=' */
-
 	while (src[length + equals] == '=') equals++;
 
 	num = (length + equals) / 4;
@@ -289,10 +287,12 @@ static int pap_authorize(void *instance, REQUEST *request)
 	int auth_type = FALSE;
 	int found_pw = FALSE;
 	int user_pw = FALSE;
-	VALUE_PAIR *vp;
+	VALUE_PAIR *vp, *next;
 	VALUE_PAIR *cleartext_pw = NULL;
 
-	for (vp = request->config_items; vp != NULL; vp = vp->next) {
+	for (vp = request->config_items; vp != NULL; vp = next) {
+		next = vp->next;
+
 		switch (vp->attribute) {
 		case PW_USER_PASSWORD: /* deprecated */
 			user_pw = TRUE;
@@ -576,7 +576,7 @@ static int pap_authenticate(void *instance, REQUEST *request)
 	switch (scheme) {
 	case PAP_ENC_CLEAR:
 	do_clear:
-		DEBUG("rlm_pap: Using clear text password.");
+		DEBUG("rlm_pap: Using clear text password \"%s\".", vp->strvalue);
 		if (strcmp((char *) vp->strvalue,
 			   (char *) request->password->strvalue) != 0){
 			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: CLEAR TEXT password check failed");
